"use strict";

var _interopRequireWildcard = require("@babel/runtime/helpers/interopRequireWildcard");

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = exports.isArrayUpToDate = exports.REMOVE_ARRAY_REDUCER = exports.UPDATE_ARRAY_REDUCER = void 0;

var _regenerator = _interopRequireDefault(require("@babel/runtime/regenerator"));

var _saveArray = _interopRequireWildcard(require("./utils/saveArray"));

var _ramdaUtils = require("./utils/ramdaUtils");

var _waitAvailability = _interopRequireDefault(require("./utils/waitAvailability"));

var _log = _interopRequireDefault(require("./utils/log"));

var UPDATE_ARRAY_REDUCER = '@@redux-pouchdb/UPDATE_ARRAY_REDUCER';
exports.UPDATE_ARRAY_REDUCER = UPDATE_ARRAY_REDUCER;
var REMOVE_ARRAY_REDUCER = '@@redux-pouchdb/REMOVE_ARRAY_REDUCER';
exports.REMOVE_ARRAY_REDUCER = REMOVE_ARRAY_REDUCER;
var initialized = {};
var running = {};

var isRunning = function isRunning(reducerName) {
  return running[reducerName] !== undefined && running[reducerName] > 0;
};

var isInitialized = function isInitialized(reducerName) {
  return initialized[reducerName] !== false;
};

var isArrayUpToDate = function isArrayUpToDate(reducerName) {
  // log(
  //   'isArrayUpToDate',
  //   isInitialized(reducerName),
  //   isUpToDate(reducerName),
  //   !isRunning(reducerName)
  // )
  return isInitialized(reducerName) && (0, _saveArray.isUpToDate)(reducerName) && !isRunning(reducerName);
};

exports.isArrayUpToDate = isArrayUpToDate;

var updateArrayReducer = function updateArrayReducer(store, doc, reducerName) {
  // log('store.dispatch update array', JSON.stringify(doc, null, 2))
  return store.dispatch({
    type: UPDATE_ARRAY_REDUCER,
    reducerName: reducerName,
    doc: doc
  });
};

var removeArrayReducer = function removeArrayReducer(store, doc, reducerName) {
  // log('store.dispatch remove array', JSON.stringify(doc, null, 2))
  return store.dispatch({
    type: REMOVE_ARRAY_REDUCER,
    reducerName: reducerName,
    doc: doc
  });
};

var initializePersistentArrayReducer = function initializePersistentArrayReducer(storeGetter, db, reducerName, saveArrayReducer) {
  var store, res;
  return _regenerator["default"].async(function initializePersistentArrayReducer$(_context) {
    while (1) {
      switch (_context.prev = _context.next) {
        case 0:
          _context.prev = 0;
          _context.next = 3;
          return _regenerator["default"].awrap((0, _waitAvailability["default"])(storeGetter));

        case 3:
          store = _context.sent;
          _context.next = 6;
          return _regenerator["default"].awrap(db.allDocs({
            include_docs: true
          }));

        case 6:
          res = _context.sent;
          _context.next = 9;
          return _regenerator["default"].awrap(Promise.all(res.rows.map(function (row) {
            return updateArrayReducer(store, row.doc, reducerName);
          })));

        case 9:
          db.changes({
            include_docs: true,
            live: true,
            since: 'now'
          }).on('change', function (change) {
            // log('change', change)
            if (change.doc) {
              // log('updateArrayReducer', change.doc)
              change.doc._deleted ? removeArrayReducer(store, change.doc, reducerName) : updateArrayReducer(store, change.doc, reducerName);
            } else {
              // log('saveArrayReducer', store.getState())
              saveArrayReducer(store.getState());
            }
          });
          _context.next = 15;
          break;

        case 12:
          _context.prev = 12;
          _context.t0 = _context["catch"](0);
          console.error(_context.t0);

        case 15:
          initialized[reducerName] = true;

        case 16:
        case "end":
          return _context.stop();
      }
    }
  }, null, null, [[0, 12]]);
}; // Higher order reducer


var persistentArrayReducer = function persistentArrayReducer(storeGetter, db, reducerName) {
  return function (reducer) {
    var lastState;
    initialized[reducerName] = false;
    var saveArrayReducer = (0, _saveArray["default"])(db, reducerName); // log(storeGetter.toString())

    initializePersistentArrayReducer(storeGetter, db, reducerName, saveArrayReducer);
    return function (state, action) {
      // log('reducer', action.type, state)
      if (action.type === UPDATE_ARRAY_REDUCER && action.reducerName === reducerName && action.doc) {
        // log('action', action)
        var found = false;
        lastState = state.map(function (item) {
          if ((0, _ramdaUtils.equalsOmittingDocProps)(item, action.doc) || item._id === action.doc._id) {
            found = true;
            return action.doc;
          }

          return item;
        });

        if (!found) {
          lastState.push(action.doc);
        }

        return reducer(lastState, action);
      } else if (action.type === REMOVE_ARRAY_REDUCER && action.reducerName === reducerName && action.doc) {
        lastState = state.filter(function (item) {
          return !(0, _ramdaUtils.equalsOmittingDocProps)(item, action.doc) && item._id !== action.doc._id;
        });
        return reducer(lastState, action);
      }

      var reducedState = reducer(state, action);

      var init = function init() {
        return _regenerator["default"].async(function init$(_context2) {
          while (1) {
            switch (_context2.prev = _context2.next) {
              case 0:
                if (!running[reducerName]) running[reducerName] = 0;
                running[reducerName] += 1;

                if (!(isInitialized(reducerName) && !(0, _ramdaUtils.equalsOmittingDocProps)(reducedState, lastState))) {
                  _context2.next = 6;
                  break;
                }

                lastState = reducedState;
                _context2.next = 6;
                return _regenerator["default"].awrap(saveArrayReducer(reducedState));

              case 6:
                running[reducerName] -= 1;

              case 7:
              case "end":
                return _context2.stop();
            }
          }
        });
      };

      init(); // log('lastState', lastState, 'reducedState', reducedState)

      return reducedState;
    };
  };
};

var _default = persistentArrayReducer;
exports["default"] = _default;